__all__ = list(
    map(
        str,
        [
            'connection',
            'activation',
            'antivirus',
            'buckets',
            'directoryservice',
            'enum',
            'login',
            'query',
            'logs',
            'portals',
            'reports',
            'remote',
            'servers',
            'devices',
            'session',
            'users',
            'cloudfs',
            'zones',
            'files',
            'decorator'
            ]
    )
)
