__all__ = list(
    map(
        str,
        [
            'connection',
            'query',
            'afp',
            'aio',
            'array',
            'audit',
            'backup',
            'cache',
            'cli',
            'config',
            'directoryservice',
            'drive',
            'enum',
            'ftp',
            'groups',
            'licenses',
            'login',
            'logs',
            'mail',
            'network',
            'nfs',
            'ntp',
            'power',
            'rsync',
            'services',
            'session',
            'shares',
            'shell',
            'smb',
            'support',
            'sync',
            'syslog',
            'telnet',
            'timezone',
            'users',
            'volumes',
            'files',
            'remote',
            'uri',
            'decorator'
        ]
    )
)
